/**************************************************************************************************
  RTC.c

  Copyright (C) 2015  Val Gretchev

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.

***************************************************************************************************
  Real Time Clock Driver
**************************************************************************************************/

// Include Files
#include "RTC.h"
#include "Buffers.h"
#include "USB_Driver3.h"



// Variables
TTime MyTime;  // Date/time structure area

unsigned long RTC_Time;
unsigned long RTC_Date;





/**************************************************************************************************
  RTC Initialize
**************************************************************************************************/
void RTC_Initialize(void) {

     RCC_APB1ENR.PWREN = 1;             // Enable power interface clock

     if (RCC_BDCR.RTCEN == 0) {         // if RTC is disabled ...
          PWR_CR.DBP = 1;               // Allow access to RTC
          RTC_WPR = 0xCA;               // Unlock write protection
          RTC_WPR = 0x53;               // Unlock write protection

          RCC_BDCR = 0x00010000;        // Make it possible to change clock source
          RCC_BDCR = 0x00008101;        // Set RTCEN, LSE, LSEON
     }

     while(RTC_ISR.RSF != 1) {}         // Wait for RTC APB registers synchronization
     while(RCC_BDCR.LSERDY != 1) {};    // Wait till LSE is ready

     RTC_WPR = 0xFF;                    // Set write protection register lock

     USB_PutQueue("RTC initialized\r\n");
}

/**************************************************************************************************
  RTC write
**************************************************************************************************/
void RTC_Write(TTime *RTCC_Time) {
     unsigned long temp;

     PWR_CR.DBP = 1;                    // Enable access to RTC, RTC backup regs, SRAM
     RTC_WPR = 0xCA;                    // Unlock write protection
     RTC_WPR = 0x53;                    // Unlock write protection

     RTC_ISR = 0x00000080;              // Set initialization mode (bit 7 of RTC_ISR)

     while(RTC_ISR.INITF != 1) {};      // Wait for INITF (bit 6 of RTC_ISR) to come on

     RTC_PRERbits.PREDIV_S = 0xFF;      // Set synchronous prescaler factor to 256
     RTC_PRERbits.PREDIV_A = 0x7F;      // Set asynchronous division factor to 128

     // Set the time register
     temp = (unsigned long)Dec2Bcd(RTCC_Time->hours) << 16;
     temp += (unsigned long)Dec2Bcd(RTCC_Time->minutes) << 8;
     temp += (unsigned long)Dec2Bcd(RTCC_Time->seconds);
     //temp += (unsigned long)(RTCC_Time->ampm) << 22;  // Not required in 24hr mode
     RTC_TR = temp;                     // Set Time register in RTC

     // Set the date register
     temp = (unsigned long)Dec2Bcd16(RTCC_Time->year) << 16;
     temp += (unsigned long)Dec2Bcd(RTCC_Time->month) << 8;
     temp += (unsigned long)Dec2Bcd(RTCC_Time->day);
     RTC_DR = temp;                     // Set date register in RTC

     RTC_CRbits.FMT = 0;                // Set FMT 24H format
     RTC_CRbits.WCKSEL = 0;             // Set RTC/16 clock

     RTC_ISR = 0x00000000;              // Exit initialization mode

     RTC_WPR = 0xFF;                    // Enable the write protection for RTC registers
}

/**************************************************************************************************
  RTC Read
**************************************************************************************************/
void RTC_Read(TTime *RTCC_Time) {

     RTC_Time = RTC_TR;                 // Read the time register
     RTC_Date = RTC_DR;                 // Read the date register

     // Extract the Time fields
     RTCC_Time->hours = Bcd2Dec((short)((RTC_Time & 0x3F0000) >> 16));
     RTCC_Time->minutes = Bcd2Dec((short)((RTC_Time & 0x007F00) >> 8));
     RTCC_Time->seconds = Bcd2Dec((short)(RTC_Time & 0x00007F));

     // Extract the date fields
     RTCC_Time->year = Bcd2Dec16((unsigned)((RTC_Date & 0xFFFF0000) >> 16));
     RTCC_Time->month = Bcd2Dec((short)((RTC_Date & 0x001F00) >> 8));
     RTCC_Time->day = Bcd2Dec((short)(RTC_Date & 0x00003F));
}

/**************************************************************************************************
  RTC Write Backup Registers
**************************************************************************************************/
void RTC_WriteSRAM(unsigned short Reg, unsigned long Value) {

     switch (Reg) {
          case  0: RTC_BKP0R  = Value; break;
          case  1: RTC_BKP1R  = Value; break;
          case  2: RTC_BKP2R  = Value; break;
          case  3: RTC_BKP3R  = Value; break;
          case  4: RTC_BKP4R  = Value; break;
          case  5: RTC_BKP5R  = Value; break;
          case  6: RTC_BKP6R  = Value; break;
          case  7: RTC_BKP7R  = Value; break;
          case  8: RTC_BKP8R  = Value; break;
          case  9: RTC_BKP9R  = Value; break;
          case 10: RTC_BKP10R = Value; break;
          case 11: RTC_BKP11R = Value; break;
          case 12: RTC_BKP12R = Value; break;
          case 13: RTC_BKP13R = Value; break;
          case 14: RTC_BKP14R = Value; break;
          case 15: RTC_BKP15R = Value; break;
          case 16: RTC_BKP16R = Value; break;
          case 17: RTC_BKP17R = Value; break;
          case 18: RTC_BKP18R = Value; break;
          case 19: RTC_BKP19R = Value; break;
          default: USB_PutQueue("Invalid reg address\r\n"); break;
     }
}

/**************************************************************************************************
  RTC Read Backup Registers
**************************************************************************************************/
unsigned long RTC_ReadSRAM(unsigned short Reg) {
     unsigned long Temp;

     switch (Reg) {
          case  0: Temp = RTC_BKP0R;  break;
          case  1: Temp = RTC_BKP1R;  break;
          case  2: Temp = RTC_BKP2R;  break;
          case  3: Temp = RTC_BKP3R;  break;
          case  4: Temp = RTC_BKP4R;  break;
          case  5: Temp = RTC_BKP5R;  break;
          case  6: Temp = RTC_BKP6R;  break;
          case  7: Temp = RTC_BKP7R;  break;
          case  8: Temp = RTC_BKP8R;  break;
          case  9: Temp = RTC_BKP9R;  break;
          case 10: Temp = RTC_BKP10R; break;
          case 11: Temp = RTC_BKP11R; break;
          case 12: Temp = RTC_BKP12R; break;
          case 13: Temp = RTC_BKP13R; break;
          case 14: Temp = RTC_BKP14R; break;
          case 15: Temp = RTC_BKP15R; break;
          case 16: Temp = RTC_BKP16R; break;
          case 17: Temp = RTC_BKP17R; break;
          case 18: Temp = RTC_BKP18R; break;
          case 19: Temp = RTC_BKP19R; break;
          default: USB_PutQueue("Invalid reg address\r\n"); Temp = 0; break;
     }
     return Temp;
}

/**************************************************************************************************
  RTC Write Coarse Calibration Register.
  Cal bit 7 = 0  -  make clock faster
  Cal bit 7 = 1  -  make clock slower
  Cal bits 4:0   -  calibration value
**************************************************************************************************/
void RTC_WriteCCR(char Cal) {

     PWR_CR.DBP = 1;                    // Enable access to RTC, RTC backup regs, SRAM
     RTC_WPR = 0xCA;                    // Unlock write protection
     RTC_WPR = 0x53;                    // Unlock write protection
     RTC_ISR = 0x00000080;              // Set initialization mode (bit 7 of RTC_ISR)
     while(RTC_ISR.INITF != 1) {};      // Wait for INITF (bit 6 of RTC_ISR) to come on
     RTC_CALIBR = Cal;                  // Set calibration register with value supplied
     RTC_ISR = 0x00000000;              // Exit initialization mode
     RTC_WPR = 0xFF;                    // Set write protection register lock
}

/**************************************************************************************************
  RTC Read Coarse Calibration Register.
**************************************************************************************************/
void RTC_ReadCCR(void) {
     char *BufNum;

     BufNum = GetBuffer();
     sprintf(BufNum,"%04X\r\n",RTC_CALIBR);
     USB_PutQueue(BufNum);
}

/**************************************************************************************************
  Set the date/time into RTC registers.
**************************************************************************************************/
void SetTime(void) {
     MyTime.ampm = 1;
     MyTime.seconds = 30;
     MyTime.minutes = 00;
     MyTime.hours = 14;
     MyTime.day = 16;
     MyTime.month = 12;
     MyTime.year = 16;
     RTC_Write(&MyTime);
     USB_PutQueue("Time and date set in RTC\r\n");
}

/**************************************************************************************************
  Get the date/time from RTC registers and display on USB port.
**************************************************************************************************/
void GetTime(void) {
     char *BufNum;

     RTC_Read(&MyTime);
     BufNum = GetBuffer();
     /*
     if (MyTime.year > 9) {
         sprintf(BufNum,"20%d/%d/%d %d:%d:%d\r\n",MyTime.year,MyTime.month,MyTime.day,MyTime.hours,
                                                  MyTime.minutes,MyTime.seconds);
     } else {
         sprintf(BufNum,"200%d/%d/%d %d:%d:%d\r\n",MyTime.year,MyTime.month,MyTime.day,MyTime.hours,
                                                  MyTime.minutes,MyTime.seconds);
     }
     */
     sprintf(BufNum,"20%02d/%02d/%02d %02d:%02d:%02d\r\n",MyTime.year,MyTime.month,MyTime.day,
                                     MyTime.hours,MyTime.minutes,MyTime.seconds);
     USB_PutQueue(BufNum);
}

/**************************************************************************************************
  Get the Time-stamp string from RTC registers and display on USB port.
**************************************************************************************************/
char* GetTimeStampStr(void){
     char TimeStamp[32] = {0};
     char *BufNum;
     //TimeStamp[0] = 0x0E; // length
     //TimeStamp[1] = 0x20; //command response
     //TimeStamp[2] = 0;
     //SetTime();
     //TimeStamp = GetBuffer();
     RTC_Read(&MyTime);
     TimeStamp[0] = 0x08; // length
////     TimeStamp[1] = TIME_STAMP; //command response
     TimeStamp[2] = MyTime.year;
     TimeStamp[3] = MyTime.month;
     TimeStamp[4] = MyTime.day;
     TimeStamp[5] = MyTime.hours;
     TimeStamp[6] = MyTime.minutes;
     TimeStamp[7] = MyTime.seconds;
      //(char) ((MyTime.year >> 8) & 0xFF); //upper 8 bits
     //TimeStamp[3] = (char) (MyTime.year & 0xFF); // lower 8 bits
     //TimeStamp[4] = 0x41;
     BufNum = GetBuffer();
    // sprintf(BufNum,"%02d/%02d/%02d %02d:%02d:%02d",MyTime.year,MyTime.month,MyTime.day,
                                     //MyTime.hours,MyTime.minutes,MyTime.seconds);
       sprintf(BufNum,"%02d/%02d/%02d %02d:%02d:%02d",TimeStamp[2],TimeStamp[3],TimeStamp[4],
               TimeStamp[5],TimeStamp[6], TimeStamp[7]);

     USB_PutQueue(BufNum);
     return TimeStamp;
}



/**************************************************************************************************
  End of File
**************************************************************************************************/