/**************************************************************************************************
  USART2_Driver.c

  Copyright (C) 2015  Val Gretchev

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.

***************************************************************************************************

  Development Board: EasyMx PRO v7 for STM32 ARM
  Microcontroller: STM32F407VGT6
  Oscillator: external 25 MHz crystal
  MPU Frequency: 120 MHz from PLL

  MPU USART2 is directly connected to mikroBUS slot 2 TX and RX pins. It also communicates with
  the FTDI Serial-to-USB (FT232RL) chip by means of SW12-3 and SW12-4 located on the board and
  comes out on USB UART B connector.

  Pin          Function     Slot 2   Switch     Signal     Connector
  (86) PD5     USART2_TX    TX       SW12-3     TX-FTDI2   USB UART B
  (87) PD6     USART2_RX    RX       SW12-4     RX-FTDI2   USB UART B

  Note: If using FTDI2 for serial communication, do not place a click board in slot 2 that
  requires TX and RX pins.


  For SYSCLK = 120 MHz and APB1 Prescaler = 4 the peripheral clock for APB1 = 30 MHz
  USART2 is on APB1 BUS and the peripheral clock is 30 MHz
  USARTDIV = Fck / 16 * baud = 30,000,000 / (16 * 9600) = 0d195.3125
  DIV_Mantissa = 0d195 = 0xC3
  DIV_Fraction = 16 * 0d0.3125 = 0d5 or 0x5
  Concatenate DIV Mantissa and DIV_Fraction,
  Therefore USART2_BRR = 0xC35
*/
/**************************************************************************************************
  USART2ForFreeRTOS.c
**************************************************************************************************/

// Include Files
#include <stdint.h>
#include <stddef.h>
#include "FreeRTOSConfig.h"
#include "FreeRTOS.h"
#include "task.h"
#include "queue.h"
#include "semphr.h"
#include "timers.h"
#include "projdefs.h"
#include "portmacro.h"

#include "Common.h"
#include "Buffers.h"
#include "USART2_driver.h"
#include "USB_Driver3.h"
#include "WiFi_ESP_Driver.h"

//#include "USART3_driver.h"
#include "HTTP.h"


// Variables
extern const code char indexPage1[];
extern const char SSID[];
extern const char Password[];




// Queues
QueueHandle_t U2RcvByteQueueHandle;    // Queue of receive bytes
QueueHandle_t U2XmtPointerQueue;       // Queue of transmit message pointers

// Semaphores
SemaphoreHandle_t U2XmtSemHandle;      // Semaphore indicating transmitter is idle



// Buffers
char U2RcvBuf[2048];                   // Receive messages are assembled in this buffer
char U2RcvBufPtr = 0;                  // Pointer to the next free byte in RcvBuf
char *U2XmtBuf;                        // Pointer to the current message being transmitted
unsigned U2XmtBufPtr;                  // Points to a byte in transmit buffer

// The receive message routine has 2 states
tPtrFcn U2_RxFuncPtr = USART2_Rx1;     // Start in State 1













/**************************************************************************************************
  USART2 interrupt handler
**************************************************************************************************/
void USART2_Interrupt() iv IVT_INT_USART2 ics ICS_AUTO {
     char R_data;        // Receive data byte
     char T_data;        // Transmit data byte
     BaseType_t xHigherPriorityTaskWoken = pdFALSE;

     if ((RXNE_USART2_SR_bit) /* || (ORE_bit) */ ) {
          R_data = USART2_DR;                // Fetch a byte from UART hardware
          xQueueSendFromISR(U2RcvByteQueueHandle,&R_data,&xHigherPriorityTaskWoken);
     }

     if ((TXE_USART2_SR_bit)&&(TXEIE_USART2_CR1_bit)) {
          T_data = U2XmtBuf[U2XmtBufPtr++];  // Get a byte from the transmit buffer
          if (T_data == NUL) {               // If it is the end of message marker
               TXEIE_USART2_CR1_bit = 0;     // Disable transmitter interrupts
               xSemaphoreGiveFromISR(U2XmtSemHandle,&xHigherPriorityTaskWoken);
          } else {
               USART2_DR = T_data;           // Transfer data byte to transmit register
          }
     }
     portYIELD_FROM_ISR(xHigherPriorityTaskWoken);
}

/**************************************************************************************************
  This routine sets the baud rate.
**************************************************************************************************/
void USART2_SetBaud(void) {

/*
//   Set registers directly
     GPIODEN_bit = 1;             // Set GPIOD clock enable bit

     GPIOD_MODER |= 0x00002800;   // Set port D MODER bits 6 and 5 to 10: alternate function mode
     GPIOD_AFRL |= 0x07700000;    // Select alternate function 0111: AF7 for bits 6 and 5

     USART2EN_bit = 1;            // Set USART2 clock enable bit
     USART2_CR1 |= 0x0000200C;    // Set UE, TE, and RE in USART2 control register 1

     USART2_BRR = 0x00000C35;     // Set baud rate register for 9600 with 30MHz on APB1
*/
//
     // Could use MikroE library. UART library must be check marked
     // Configure USART2 for 9600 baud, 8-bits, no parity, 1 stop bit on pins PD5 (TX) and PD6 (RX)
     UART2_Init_Advanced(9600, _UART_8_BIT_DATA, _UART_NOPARITY, _UART_ONE_STOPBIT, &_GPIO_MODULE_USART2_PD56);
//
     NVIC_IntEnable(IVT_INT_USART2); // Set interrupt vector
     NVIC_SetIntPriority(IVT_INT_USART2, _NVIC_INT_PRIORITY_LVL4);
}

/**************************************************************************************************
  This routine initializes 2 queues: a byte queue and a message queue.
**************************************************************************************************/
void USART2_RxQueueInit(void) {
     U2RcvByteQueueHandle = xQueueCreate(U2_RCV_BYTE_Q_LENGTH, sizeof(char));
     if (U2RcvByteQueueHandle == NULL) {
         USB_PutQueue("Error: Could not create USART2 receive byte queue\r\n");
     } else {
         USB_PutQueue("Diag: USART2 receive byte queue created successfully\r\n");
         vQueueAddToRegistry(U2RcvByteQueueHandle, "U2 RcvByte Queue");
     }
}

/**************************************************************************************************
  This routine starts the receiver.
**************************************************************************************************/
void USART2_StartRx(void) {
     RXNEIE_USART2_CR1_bit = 1;        // Interrupt when receive buffer is full (RXNE=1)
}

/**************************************************************************************************
  USART2 receiver initialization
**************************************************************************************************/
void USART2_RcvInit(void) {
     USART2_SetBaud();
     USART2_RxQueueInit();
     USART2_StartRx();
}

/**************************************************************************************************
  UART_Rx State Machine. Call this function in the main loop.
**************************************************************************************************/
void USART2_Rx(void) {
     U2_RxFuncPtr();    // Use the function pointer to go to a state
}

/**************************************************************************************************
  USART2 receive subroutine
**************************************************************************************************/
void USART2_Rx1(void) {
     char RxChar;

     xQueueReceive(U2RcvByteQueueHandle, &RxChar, 0xFFFFFFFFU); // Block here

     switch (RxChar) {
          case NUL:         // Discard NUL character
               break;

          case '\r':        // Use return character
               U2RcvBuf[U2RcvBufPtr++] = RxChar;  // Put received character into buffer
               /*
               U2RcvBuf[U2RcvBufPtr] = 0;         // End marker
               U2RcvBufPtr = 0;                   // Point to start of buffer
               U2_RxFuncPtr = USART2_Rx2;          // Go to state 2
               */
               break;

          case '\n':        // Use new line character
               //
               U2RcvBuf[U2RcvBufPtr++] = RxChar;  // Put received character into buffer
               U2RcvBuf[U2RcvBufPtr] = 0;         // End marker
               U2RcvBufPtr = 0;                   // Point to start of buffer
               U2_RxFuncPtr = USART2_Rx2;          // Go to state 2
               //
               break;

          default:
               if (U2RcvBufPtr <  sizeof(U2RcvBuf)-2)     // Prevents buffer overrun
               U2RcvBuf[U2RcvBufPtr++] = RxChar;       // Put character in buffer
               break;
     }
}

/**************************************************************************************************
  UART_Rx State 2: Execute the command indicated by the first byte in the buffer then return to
  State 1.
**************************************************************************************************/
void USART2_Rx2(void) {
     char *BufNum;
     char Str[4];

     switch (U2RcvBuf[0]) {

          // Change to suit application

          case 'a':  // Diagnostic test for USART2
               USART2_PutQueue("Command 'a' received\r\n");
               break;
          case 'b':  // This is a test to displays the web server commands on USART2
               TestForHTTP();
               break;
          case 'c':  // Comment out USB_PutQueue and uncomment USART2_PutQueu in WiFi_ESP_Driver.c
               SendLength((char*)indexPage1);
               break;
          case 'd':  // Display sizef indexPage1
               DisplaySizePage1();
               //DisplaySizePage2();
               //DisplaySizePage3();
               break;
          case 'e':  // Display strlen indexPage1
               DisplayPageSize(indexPage1);
               //DisplayPageSize(indexPage2);
               //DisplayPageSize(indexPage3);
               break;
          case 'f': // Test
               BufNum = GetBuffer();
               sprintf(BufNum,"AT+CWJAP_DEF=\"");
               strcat(BufNum,(char*)SSID);
               strcat(BufNum,"\",\"");
               strcat(BufNum,(char*)Password);
               strcat(BufNum,"\"\r\n");
               USART2_PutQueue(BufNum);
               break;
          case 'g':
               break;
          case 'h':
               break;
          case 'i':
               break;
          case 'j':
               break;
          case 'k':
               break;
          case 'l':
               break;
          case 'm':
               break;
          case 'n':
               break;
          case 'o':
               break;
          case 'p':
               break;
          case 'q':
               break;
          case 'r':
               break;
          case 's':
               break;
          case 't':
               break;
          case 'u':
               break;
          case 'v':
               break;
          case 'w':
               break;
          case 'x':
               break;
          case 'y':
               break;
          case 'z': // Force a system reset
               SystemReset();
               break;

          default:
               USART2_PutQueue("Error: Unknown command: ");

               BufNum = GetBuffer();    // Get a free buffer
               (void)sprintf(BufNum,"%c\r\n",U2RcvBuf[0]);
               USART2_PutQueue(BufNum);  // Send the message
               break;
     }
     U2_RxFuncPtr = USART2_Rx1;     // Go back to state 1
}

/**************************************************************************************************
  USART2 receive task
**************************************************************************************************/
//
void USART2_RcvTask(void *pvParameters) {

     USART2_RcvInit();

     while (1) {
          USART2_Rx();    // Check for incoming USART2 messages

          // Delay
          //vTaskDelay(pdMS_TO_TICKS(50)); // Delay for 50ms
     }
}
//
/*************************************************************************************************/










/**************************************************************************************************
  USART2 put queue.
**************************************************************************************************/
void USART2_PutQueue(char *msg) {
     BaseType_t rtnCode;

     rtnCode = xQueueSend(U2XmtPointerQueue, &msg, 0); // No blocking

     if (rtnCode == pdFAIL)
          USB_PutQueue("Error: Failed to enter message pointer into USART2 queue\r\n");
}

/**************************************************************************************************
  This routine starts the transmitter.
**************************************************************************************************/
void USART2_StartTx(void) {
     TXEIE_USART2_CR1_bit = 1;         // Interrupt when transmit buffer is empty (TXE=1)
}

/**************************************************************************************************
  This routine enters a pointer to a text message and starts the transmitter.
**************************************************************************************************/
void USART2_SendMsg(char *s) {
     U2XmtBuf = s;            // Pointer to the message for the interrupt handler
     U2XmtBufPtr = 0;         // Set character counter to start of text message
     USART2_StartTx();         // Start the transmitter
}

/**************************************************************************************************
  This routine checks the transmit buffer queue and if it is not empty, it extracts the pointer
  to the message and sets up the transmitter. Call this function in the main loop.
**************************************************************************************************/
void USART2_Tx(void) {
     char *pcReceivedAddr;    // Last pointer to message obtained from pointer queue
     BaseType_t rtnCode;      // Function return code

     xSemaphoreTake(U2XmtSemHandle, portMAX_DELAY); // Block here
     rtnCode = xQueueReceive(U2XmtPointerQueue, &pcReceivedAddr, 0xFFFFFFFFU); // Block here

     USART2_SendMsg(pcReceivedAddr);
}

/**************************************************************************************************
  USART2 transmit task
**************************************************************************************************/
//
void USART2_XmtTask(void *pvParameters) {

     U2XmtSemHandle = xSemaphoreCreateBinary(); // Semaphore indicating transmitter is idle
     if (U2XmtSemHandle == NULL) {
         USB_PutQueue("Error: Could not create USART2 transmit semaphore\r\n");
     } else {
         USB_PutQueue("Diag: USART2 transmit semaphore created successfully\r\n");
         vQueueAddToRegistry(U2XmtSemHandle, "USART2 transmit Semaphore");
     }


     U2XmtPointerQueue = xQueueCreate(64, sizeof(char *)); // Queue of message pointers
     if (U2XmtPointerQueue == NULL) {
         USB_PutQueue("Error: Could not create USART2 transmit pointer queue\r\n");
     } else {
         USB_PutQueue("Diag: USART2 transmit pointer queue created successfully\r\n");
         vQueueAddToRegistry(U2XmtPointerQueue, "USART2 pointer queue");
     }

     xSemaphoreGive(U2XmtSemHandle); // This gets us past the semaphore block initially
     USART2_PutQueue("USART2 task started\r\n");
     USB_PutQueue("Diag: USART2 task started\r\n");

     while (1) {
          USART2_Tx();    // Check for outgoing USART2 messages

          // Delay
          //vTaskDelay(pdMS_TO_TICKS(50)); // Delay for 50ms
     }
}
//

/**************************************************************************************************
  End of File
**************************************************************************************************/