/**************************************************************************************************
  USB_Driver3.c

  Copyright (C) 2015  Val Gretchev

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.

***************************************************************************************************
  USB HID Driver for FreeRTOS
  The transmit task blocks when the transmit queue is empty. There was no way to make the receive
  task blocking since the interrupt fires all the time. Therefore, the receive USB task runs
  continuously.

  Development Board: EasyMx PRO v7 for STM32 ARM
  Microcontroller: STM32F407VGT6
  Oscillator: external 25 MHz crystal
  MPU Frequency: 120 MHz from PLL

  SW12 bit 7 (VBUS) must be set to the right.

  The USB connector is serviced by the internal USB driver and is configured as USB OTG Full Speed.

  PA11 Pin 70 (USB_OTG_FS_DM)
  PA12 Pin 71 (USB_OTG_FS_DP)
  PA9  Pin 68 (USB_OTG_FS_VBUS)

  The external crystal is 25MHz and HSE is divided by 15 and multiplied by 144 giving a frequency
  of 240MHz. This is further divided by 5 in the Main PLL to produce 48MHz for USB.

  On my system, it tales about 11 seconds for USB to innumerate. That is the reason for the
  14-second delay in the start-up function.

**************************************************************************************************/

// Include Files
#include <stdint.h>
#include <stddef.h>
#include "FreeRTOSConfig.h"
#include "FreeRTOS.h"
#include "task.h"
#include "queue.h"
#include "semphr.h"
#include "timers.h"
#include "projdefs.h"
#include "portmacro.h"

#include "Common.h"
#include "USB_Driver3.h"
#include "Buffers.h"
#include "USART3_driver.h"
#include "USART2_driver.h"
#include "WiFi_ESP_Driver.h"
#include "HTTP.h"
//#include "MyTimers.h"
//#include "RTC.h"


// Buffers
char USB_RdBuff[64];               // USB HID messages are limited to 64 bytes
char USB_WrBuff[64];

// Queues
QueueHandle_t xPointerQueue;       // Queue of pointers

// Semaphores
SemaphoreHandle_t usbSemHandle;    // Used for testing only

// Variables
short USB_Initialized;
//TTime DateTimeSet;                 // Date/time structure area

// Test structure
struct AMessage *TestStruct, TestOne;


extern char htmlPage[];
//extern const code char indexPage1[];
//extern const code char indexPage2[];
//extern char DNS_SErverIP[];



extern const code char indexPage1[];




/**************************************************************************************************
   This is the USB interrupt handler
**************************************************************************************************/
void USBInterrupt() iv IVT_INT_OTG_FS ics ICS_AUTO {

     USB_Interrupt_Proc();    // USB interrupt fires all the time. Can't set semaphore here.
}

/**************************************************************************************************
   Hardware Initialization
**************************************************************************************************/
void USB_Initialize(void) {

     HID_Enable(&USB_RdBuff,&USB_WrBuff);

     // Set interrupt priority
     NVIC_SetIntPriority(IVT_INT_OTG_FS, _NVIC_INT_PRIORITY_LVL4);

     USB_Initialized = 1;
}

/**************************************************************************************************
   This routine checks USB for a receive message then decodes the first character for a command.
**************************************************************************************************/

// Call this function in main loop
void USB_CheckIn(void) {
     char *BufNum;
     char Str[16];
     char LengthVal[8];
     int  Length;
     short RstVal;

     //xSemaphoreTake(usbSemHandle,portMAX_DELAY);  // No place to set the semaphore.

     if (HID_Read() != 0) {
          switch (USB_RdBuff[0]) { // The first character in read buffer is a command
               case 'a': // Diagnostic test of USB
                    USB_PutQueue("Command 'a' received\r\n");
                    break;
               case 'b': // Send software reset to WiFiESP
                    WiFiESP_PutQueue(SEND_SW_RST);
                    break;
                case 'c': // Clear flash parameters and set factory defaults into WiFiESP
                    WiFiESP_PutQueue(SET_FACT_DEFAULTS);
                    break;
               case 'd': // Set flash with wanted defaults
                    WiFiESP_PutQueue(CONFIG_FLASH);
                    break;
               case 'e': // Configure the UART of the WiFiESP and save in flash
                    WiFiESP_PutQueue(SET_UART_CONFIG);
                    break;
               case 'f': // Set AT command for a test. Replys with OK.
                    WiFiESP_PutQueue(SEND_AT);
                    break;
               case 'g': // Get URL transleted to IP addresses from DNS server
                    WiFiESP_PutQueue(REQUEST_DNS_SERVICE);
                    break;
               case 'h': // Get date and time from NTP server and set RTC
                    WiFiESP_PutQueue(REQUEST_NTP_SERVICE);
                    break;
               case 'i': // Start HTTP server and wait for an external browser connection
                    WiFiESP_PutQueue(START_HTTP_SERVER);
                    break;
               case 'j': // Start a UDP client and connect to a waiting server
                    WiFiESP_PutQueue(START_UDP_CLIENT);
                    break;
               case 'k': // Start a TCP client and connect to a waiting server
                    WiFiESP_PutQueue(START_TCP_CLIENT);
                    break;
               case 'l': // Start a UDP client in transparent mode. Good for sending binary data.
                    WiFiESP_PutQueue(START_UDP_PASSTHROUGH);
                    break;
               case 'm': // List all available access points
                    WiFiESP_PutQueue(LIST_AVAIL_APS);
                    break;
               case 'n':

                    break;
               case 'o':
                    //USART3_PutQueueText("AT+CIPDNS_CUR=1,\"208.67.222.222\"\r\n");
                    break;
               case 'p':
                    USART3_PutQueueText("AT+CWDHCPS_CUR?\r\n");
                    break;
               case 'q':
                    //USART2_PutQueue("AT+CWDHCPS_CUR=1,3,\"192.168.4.10\",\"192.168.4.15\"\r\n");
                    USART3_PutQueueText("AT+CWDHCPS_CUR=1,3,\"192.168.4.10\",\"192.168.4.15\"\r\n");
                    break;
               case 'r':
                    USART3_PutQueueText("AT+CWDHCP_CUR?\r\n");
                    break;
               case 's':
                    USART3_PutQueueText("AT+CWLIF\r\n");
                    break;
               case 't':
                    USART3_PutQueueText("AT+UART_CUR?\r\n");
                    break;
               case 'u': // This is really weird
                    TestStruct = &TestOne;
                    strcpy(Str,"Hello");
                    TestStruct->MsgLen = strlen(Str);
                    TestStruct->TheMessage = &Str;
                    BufNum = GetBuffer();
                    sprintf(BufNum,"Length: %d\r\n",TestStruct->MsgLen);
                    USB_PutQueue(BufNum);
                    BufNum = GetBuffer();
                    sprintf(BufNum,"Message address: %d\r\n",TestStruct->TheMessage);
                    USB_PutQueue(BufNum);
                    BufNum = GetBuffer();
                    sprintf(BufNum,"Message: %s\r\n",TestStruct->TheMessage);
                    USB_PutQueue(BufNum);

                    break;
               case 'v': // This works too
                    TestStruct = &TestOne;
                    TestStruct->MsgLen = strlen("Hello");
                    TestStruct->TheMessage = "Hello";
                    BufNum = GetBuffer();
                    sprintf(BufNum,"Length: %d\r\n",TestStruct->MsgLen);
                    USB_PutQueue(BufNum);
                    BufNum = GetBuffer();
                    sprintf(BufNum,"Message address: %d\r\n",TestStruct->TheMessage);
                    USB_PutQueue(BufNum);
                    BufNum = GetBuffer();
                    sprintf(BufNum,"Message: %s\r\n",TestStruct->TheMessage);
                    USB_PutQueue(BufNum);
                    break;
               case 'w':
                    BufNum = GetBuffer();
                    sprintf(BufNum,"Size of htmlPage is: %d\r\n",strlen(htmlPage));
                    USB_PutQueue(BufNum);
                    break;
               case 'x':
                    SendLength((char*)indexPage1);
                    break;
               case 'y':
                    GetLocalIP();
                    break;

               case 'z': // Force a system reset
                    SystemReset();
                    break;

                    // Etc.

               default:  // Catch all other unimplemented commands
                    BufNum = GetBuffer();  // Get a free buffer
                    sprintf(BufNum,"Unknown command: %c\r\n",USB_RdBuff[0]);
                    USB_PutQueue(BufNum);
                    break;
          }
     }
}

/**************************************************************************************************
   This routine makes an entry into the transmit queue FIFO.
**************************************************************************************************/
void USB_PutQueue(char *msg) {
     BaseType_t rtnCode;

     rtnCode = xQueueSend(xPointerQueue, &msg, 0); // No blocking

     if (rtnCode == pdFAIL)
          ;//Timer8_Ch2_Start(); // Flash PC7 LED if pdFAIL
}

/**************************************************************************************************
   This routine waits for a pointer from the transmit queue FIFO, then sends the message to USB.
**************************************************************************************************/

// Call this function in main loop
void USB_CheckOut(void) {
     char RetryCount = 0;          // Set to zero on entry to this function
     BaseType_t rtnCode;           // Function return code
     char *pcReceivedAddr;         // Last pointer to message obtained from pointer queue

     rtnCode = xQueueReceive(xPointerQueue, &pcReceivedAddr, 0xFFFFFFFFU); // Block here

     if (rtnCode == pdPASS) {
          while (((HID_Write(pcReceivedAddr,64) == 0)) && (RetryCount < 5)) { // Send 64 bytes to USB
               RetryCount++;       // Try 5 times
          }
     } else {
          //Timer8_Ch2_Start();      // Flash PC7 LED if pdFAIL
     }
}

/**************************************************************************************************
  USB receive driver task
**************************************************************************************************/
//
void usbRcvTask(void *pvParameters) {

     USB_Initialize();             // Initialize USB hardware

     // This is used for a test
     usbSemHandle = xSemaphoreCreateBinary();
     vQueueAddToRegistry(usbSemHandle, "USB receive Semaphore");

     while (1) {
          USB_CheckIn(); // Check for incoming USB messages
          //vTaskDelay(pdMS_TO_TICKS(100));
     }
}
//

/**************************************************************************************************
  USB transmit driver task
**************************************************************************************************/
//
void usbXmtTask(void *pvParameters) {

     // Create a queue of pointers
     xPointerQueue = xQueueCreate(64, sizeof(char *));
     if (xPointerQueue == NULL) {
         USB_PutQueue("Error: Could not create USB pointer queue\r\n");
     } else {
         USB_PutQueue("Diag: USB pointer queue created successfully\r\n");
         vQueueAddToRegistry(xPointerQueue, "USB message pointer queue");
     }

     //USB_PutQueue(" ");                     // Output blank line
     //RTC_Initialize();
     USB_PutQueue("Diag: USB task started\r\n");       // Send opening message on USB

     // Wait for USB to enumerate
     vTaskDelay(pdMS_TO_TICKS(14000));


     // Start diagnostic timer for PC7 LED
     //Timer8_Initialize();
     //Timer8_Ch2_Start();

     while (1) {
          USB_CheckOut(); // Check for outgoing USB messages
     }
}
//


/**************************************************************************************************
  End of File
**************************************************************************************************/